import java.util.List;

/**
 * @author ericfouh
 */
public interface IAutocomplete
{

    /**
     * Adds a new word with its associated weight to the Trie
     * 
     * @param word the word to be added to the Trie
     * @param weight the weight of the word
     */
    public void addWord(String word, long weight);


    /**
     * Initializes the Trie
     *
     * @param filename the file to read all the autocomplete data from each line
     *                 contains a word and its weight This method will call the
     *                 addWord method
     * @param k the maximum number of suggestions that should be displayed 
     */
    public void buildTrie(String filename, int k);

    
    /**
     * @return k the maximum number of suggestions that should be displayed
     */
    public int numberSuggestions();


    /**
     * @param prefix a string that can contain any character
     * @return the number of words that start with prefix.
     * @throws IllegalArgumentException if the specified prefix is null or if the prefix contains
     * characters that are not lowercase letters
     */
    public int countPrefixes(String prefix);


    /**
     * @param prefix a string that can contain any character
     * @return a List containing all the ITerm objects with query starting with
     *         prefix. Return an empty list if there are no ITerm object starting
     *         with prefix.
     * @throws IllegalArgumentException if the specified prefix is null or if the prefix contains
     * characters that are not lowercase letters
     */
    public List<ITerm> getSuggestions(String prefix);

}
