import java.util.*;

/**
 * Skeletal implementation of a {@link Map}.
 *
 * @param <K> the type of keys maintained by this map
 * @param <V> the type of mapped values
 * @author davix
 * @author gabsmi
 */
public interface IHashMap<K, V> {

    /**
     * Returns an {@link Iterator} that visits all the {@link Map.Entry}s in the {@link Map}. The
     * iterator reflects the entries in the map when it was called. This {@link Iterator} does not
     * support element removal.
     * <p>
     * If the map is modified after the {@link Iterator} is generated, the behavior of further
     * iteration on the iterator is undefined.
     *
     * @return an iterator over all the map entries
     */
    public Iterator<Map.Entry<K, V>> entryIterator();

    public int size();

    public  boolean containsValue(Object value);

    public boolean containsKey(Object key);

    public V get(Object key);

    public V put(K key, V value);

    public V remove(Object key);

    public boolean isEmpty();

    public void clear();

    public Set<Map.Entry<K, V>> entrySet();

}
