import java.util.List;

/**
 * Interface for the Trie (previously called TrieMap)
 *
 * @author gabsmi
 */
public interface ITrie<V> {

    /**
     * Returns the value to which the specified key is mapped, or {@code null} if this map contains
     * no mapping for the key.
     *
     * @param key the key whose associated value is to be returned
     * @return the value to which the specified key is mapped, or {@code null} if this map contains
     * no mapping for the key
     * @throws IllegalArgumentException if the specified key is null or if the key contains
     * characters that are not lowercase letters
     */
    public V get(CharSequence key);

    /**
     * @throws IllegalArgumentException if either the specified key or value is null
     * or if the key contains characters that are not lowercase letters
     */

    public V put(CharSequence key, V value);

    /**
     * Returns {@code true} if this map contains a mapping for the specified key.
     *
     * @param key key whose presence in this map is to be tested
     * @return {@code true} if this map contains a mapping for the specified key
     * @throws IllegalArgumentException if the specified key is null or if the key contains
     * characters that are not lowercase letters
     */
    public boolean containsKey(CharSequence key);

    /**
     * Returns {@code true} if this map maps one or more keys to the specified value.
     *
     * @throws IllegalArgumentException if the specified value is null
     */
    public boolean containsValue(Object value);

    /**
     * Removes the mapping for a key from this map if it is present.
     *
     * @param key key whose mapping is to be removed from the map
     * @return the previous value associated with the key, or {@code null} if there was no mapping
     * for the key.
     * @throws IllegalArgumentException if the specified key is null or if the key contains
     * characters that are not lowercase letters
     */
    public V remove(CharSequence key);

    /**
     * Returns the number of values whose keys have prefix as a prefix
     * Duplicate values should be double counted (see write up)
     *
     * @param prefix possible prefix of some keys in the trie
     * @return the number of entries whose key has prefix as a prefix
     * @throws IllegalArgumentException if the specified prefix is null or if the prefix contains
     * characters that are not lowercase letters
     */
    public int countPrefixes(CharSequence prefix);

    /**
     * Returns a list of values of all entries whoes key has prefix as a prefix
     * Duplicate values should not be removed (see write up)
     *
     * @param prefix possible prefix of some keys in the trie
     * @return the value of all entries whose key has prefix as a prefix
     * @throws IllegalArgumentException if the specified prefix is null or if the prefix contains
     * characters that are not lowercase letters
     */
    public List<V> allValuesWithPrefix(CharSequence prefix);
}

